package com.perforce.jdev;
import com.perforce.jdev.res.Icons;
import com.perforce.jdev.res.Overlays;

import javax.swing.Icon;
import javax.swing.KeyStroke;

import oracle.ide.controls.OverlayIcon;

import oracle.ide.scm.SCMControlItem;
import oracle.ide.scm.SCMFile;
import oracle.ide.scm.SCMFileStatus;
import oracle.ide.scm.SCMOptions;

import oracle.ide.scm.error.SCMException;

import oracle.ide.scm.op.SCMSingleFileOperation;

/**
 * Opens one file.
 *
 * @author Paul Krause
 * @version $Revision: #5 $ $Date: 2002/11/08 $
 */
public class P4Edit1Cmd extends SCMSingleFileOperation implements SCMControlItem
{
   /** Addition overlay on edit icon. */
   public static final Icon ICON =
      new OverlayIcon(Icons.CHECKOUT, Overlays.ADDED);

   /**
    * Opens a file.
    *
    * @param f a <code>SCMFile</code> to open
    * @param opts any <code>SCMOptions</code>
    * @exception SCMException if an error occurs
    */
   public void execute(SCMFile f, SCMOptions opts) throws SCMException {
      final P4File stat = (P4File) f.getStatus();
      if (stat.isControlled()) {
         stat.openForEdit(opts);
      }
      else {
         stat.openForAdd(opts);
      }
   }

   /**
    * A file can be opened if it has a proper depot path and is not
    * open already.
    *
    * @param f the active <code>SCMFile</code>
    * @return <code>true</code> if file can be opened
    * @exception SCMException if an error occurs
    */
   public boolean filter(SCMFile f) throws SCMException {
      final SCMFileStatus stat = f.getStatus();
      return (stat instanceof P4File) &&
         (!((P4File) stat).isControlled() ||
          !((P4File) stat).isOpen());
   }

   public SCMControlItem getControlItem() {
      return this;
   }

   public String getName() {
      return "Open for Edit/Add";
   }

   public char getMnemonic() {
      return 'o';
   }

   public KeyStroke getAccelerator() {
      return KeyStroke.getKeyStroke("control meta shift o");
   }

   public Icon getIcon() {
      return ICON;
   }

   public boolean isContextItem() {
      return true;
   }
}
