/* $Id: //guest/paul_krause/perforce/integrations/jdev/src/com/perforce/jdev/P4Project.java#2 $
 * Copyright 2000-2002 SoundBite Corporation.   All Rights Reserved.
 */
package com.perforce.jdev;
import java.util.Iterator;
import java.util.Vector;
import java.io.File;

import oracle.ide.scm.SCMFile;

import com.perforce.api.Env;
import com.perforce.api.FileEntry;

/**
 * A bridge between SCMFile project and P4File's concept of project.
 * 
 * In the OTN JDeveloper 
 * <a href="http://www.oracle.com/forums/message.jsp?id=850222">Forum</a>,
 * <a mailto="brian.duff@oracle.com">Brian Duff</a> writes: <quote> I've
 * logged ER 2259776 against the SCM addin API to support project-level SCM 
 * settings; we agree this would be a useful addition to the framework. 
 * You can track this ER on metalink. </quote>
 *
 * @author Paul Krause
 * @created Fri May 3 17:47:27 2002
 * @version $Revision: #2 $
 */
public class P4Project extends P4File {

   /** Perforce file specificier (pattern) for the files in this project. */
   private final String p4Spec;

   /** Description of the Field */
   private final Env p4Env;

   /**
    * Constructor for the P4Project object
    *
    * @param f The controlled .jpr file
    * @param env Perforce environment
    */
   public P4Project(SCMFile f, Env env)
   {
      super(f, env);
      final String dir = f.toFile().getParent();
      p4Spec = dir + File.separator + "...";
      p4Env = env;
   }

   /**
    * Gets the value of the project property
    *
    * @return The Project value
    */
   public boolean isProject()
   {
      return true;
   }

   /**
    * Gets the value of the open property
    *
    * @return The Open value
    */
   public boolean isOpen()
   {
      //TODO: implement this oracle.ide.model.Node method
      return false;
   }

   /**
    * Gets the value of the project sources property
    *
    * @return The ProjectSources value
    */
   public Iterator getProjectSources()
   {
      final Vector v = FileEntry.getFiles(p4Env, p4Spec);
      final Iterator vi = v.iterator();
      return
         new Iterator() {
            public boolean hasNext()
            {
               return vi.hasNext();
            }

            public Object next()
            {
               return new P4File((FileEntry) vi.next());
            }

            public void remove()
            {
               vi.remove();
            }
         };
   }

}
