package com.perforce.jdev.cm;

import com.perforce.jdev.P4PropertySheet;

import java.util.Map;

import oracle.ide.scm.util.SCMEnvironmentVars;

import oracle.jdeveloper.cm.ConnectionType;

/**
 * Part of the ConnectionManager pattern.
 *
 * @author Paul Krause
 * @version $Revision: #2 $ $Date: 2002/10/08 $
 */
public class P4ConnectionType implements ConnectionType  {

   /**
    * Connection class of this connection type, per by the Connection Manager.
    * Use this to get connection names by class.  For example,
    * <pre>
    * ConnectionManager cm;
    * String[] conn = cm.getConnectionNamesForClass(P4ConnectionType.CLASS);
    * </pre>
    * @see #getConnectionClassName
    */
   public static final String CLASS = P4ConnectionType.class.getName();

   /**
    * The name of this connection type.  Used to construct default connection.
    * @see #getTypeName
    */
   public static final String TYPE = P4ConnectionType.class.getName();


   public String getConnConfigPanelClass() {
      return P4PropertySheet.class.getName();
   }

   public String getFactoryClass() {
      return P4ConnectionFactory.class.getName();
   }

   public String getName() {
      return "Perforce Connection";
   }

   /** ConnectionType attribute. */
   public String getTypeName() {
      return getClass().getName();
   }

   public String getConnectionClassName() {
      return CLASS;
   }

   public String getDefaultUsername() {
      final Map env = SCMEnvironmentVars.getInstance().getVariables();
      final String user = env.containsKey("P4USER") ?
         (String) env.get("P4USER") : System.getProperty("user.name");
      return user;
   }

   public String getDefaultPassword() {
      final Map env = SCMEnvironmentVars.getInstance().getVariables();
      return (String) env.get("P4PASSWD");
   }

}
